#define __KERNEL__
#define MODULE
#include <linux/module.h>
#include <linux/fs.h>
#include <linux/ide-tivo.h>

#define DEVICE_NAME		"/dev/hda"

struct lowlevel_operations def_llop;
struct lowlevel_operations* llop;

int noscramble_readsectors(struct file *filp, struct FsIovec *buf, int buf_len,
	const struct FsIoRequest *request, struct io_scramble *scramble);
int noscramble_writesectors(struct file *filp, struct FsIovec *buf, int buf_len,
	const struct FsIoRequest *request, struct io_scramble *scramble);


int init_module()
{
	struct inode* inode;
	struct file file;
	int error;

	error = open_namei(DEVICE_NAME, 0, 0, &inode, NULL);
	if (error)
	{
		printk("cant open %s because %d!\n", DEVICE_NAME, -error);
		goto out;
	}

	error = -EINVAL;
	if (!inode->i_op)
	{
		printk("no inode ops on %s!\n", DEVICE_NAME);
		goto iput_and_out;
	}

	error = -EINVAL;
	if (!inode->i_op->default_file_ops)
	{
		printk("no default_file_ops on %s!\n", DEVICE_NAME);
		goto iput_and_out;
	}

	error = -EINVAL;
	if (!inode->i_op->default_file_ops->open)
	{
		printk("no open on %s!\n", DEVICE_NAME);
		goto iput_and_out;
	}

	memset(&file, 0, sizeof(struct file));
	file.f_inode = inode;

	error = inode->i_op->default_file_ops->open(inode, &file);
	if (error)
	{
		printk("open failed on %s because %d!\n", DEVICE_NAME, -error);
		goto iput_and_out;
	}

	llop = file.f_llop;

	if (file.f_op->release)
		file.f_op->release(inode, &file);

	error = -EINVAL;
	if (!llop)
	{
		printk("no low level ops!\n");
		goto iput_and_out;
	}

	error = 0;

	def_llop.readsectors = llop->readsectors;
	def_llop.writesectors = llop->writesectors;

	llop->readsectors = noscramble_readsectors;
	llop->writesectors = noscramble_writesectors;

iput_and_out:
	iput(inode);
out:
	return error;
}

void cleanup_module()
{
	llop->readsectors = def_llop.readsectors;
	llop->writesectors = def_llop.writesectors;
}

int noscramble_readsectors(struct file *filp, struct FsIovec *buf, int buf_len,
	const struct FsIoRequest *request, struct io_scramble *scramble)
{
	int result;

	MOD_INC_USE_COUNT;

/*	if (scramble)
	{
		printk("read scramble sector %lu magic %08x version %lu spt %lu roff %lu wyrd0 %08x wyrd1 %08x wyrd2 %08x wyrd3 %08x\n",
			request->sector, scramble->magic, scramble->version, scramble->sectorsPerTransfer, scramble->recordOffset,
			scramble->config.wyrd[0], scramble->config.wyrd[1], scramble->config.wyrd[2], scramble->config.wyrd[2]);
	}
*/
	result = def_llop.readsectors(filp, buf, buf_len, request, NULL);

	MOD_DEC_USE_COUNT;

	return result;
}

int noscramble_writesectors(struct file *filp, struct FsIovec *buf, int buf_len,
	const struct FsIoRequest *request, struct io_scramble *scramble)
{
	int result;

	MOD_INC_USE_COUNT;

/*	if (scramble)
	{
		printk("write scramble sector %lu magic %08x version %lu spt %lu roff %lu wyrd0 %08x wyrd1 %08x wyrd2 %08x wyrd3 %08x\n",
			request->sector, scramble->magic, scramble->version, scramble->sectorsPerTransfer, scramble->recordOffset,
			scramble->config.wyrd[0], scramble->config.wyrd[1], scramble->config.wyrd[2], scramble->config.wyrd[2]);
	}
*/
	result = def_llop.writesectors(filp, buf, buf_len, request, NULL);

	MOD_DEC_USE_COUNT;

	return result;
}


